@@ -84,6 +84,7 @@ struct uwsgi_option uwsgi_mono_options[] = {
         {"mono-assembly", required_argument, 0, "load the specified main assembly (default: uwsgi.dll)", uwsgi_opt_set_str, &umono.assembly_name, 0},
         {"mono-exec", required_argument, 0, "exec the specified assembly just before app loading", uwsgi_opt_add_string_list, &umono.exec, 0},
         {"mono-index", required_argument, 0, "add an asp.net index file", uwsgi_opt_add_string_list, &umono.index, 0},
+	{0, 0, 0, 0, 0, 0, 0},
 };
 
 static MonoString *uwsgi_mono_method_GetFilePath(MonoObject *this) {
@@ -542,7 +543,7 @@ static int uwsgi_mono_request(struct wsgi_request *wsgi_req) {
 
         wsgi_req->app_id = uwsgi_get_app_id(NULL, key, key_len, mono_plugin.modifier1);
         // if it is -1, try to load a dynamic app
-        if (wsgi_req->app_id == -1) {
+        if (wsgi_req->app_id == -1 && key_len > 0) {
         	if (uwsgi.threads > 1) {
                 	pthread_mutex_lock(&umono.lock_loader);
                 }
@@ -561,10 +562,15 @@ static int uwsgi_mono_request(struct wsgi_request *wsgi_req) {
 
 
         if (wsgi_req->app_id == -1) {
-        	uwsgi_500(wsgi_req);
-                uwsgi_log("--- unable to find Mono/ASP.NET application ---\n");
-                // nothing to clear/free
-                return UWSGI_OK;
+		if (!uwsgi.no_default_app && uwsgi.default_app > -1 && uwsgi_apps[uwsgi.default_app].modifier1 == mono_plugin.modifier1) {
+               		wsgi_req->app_id = uwsgi.default_app;
+                }
+		else {
+        		uwsgi_500(wsgi_req);
+                	uwsgi_log("--- unable to find Mono/ASP.NET application ---\n");
+                	// nothing to clear/free
+                	return UWSGI_OK;
+		}
         }
 
         struct uwsgi_app *app = &uwsgi_apps[wsgi_req->app_id];
