@@ -25,11 +25,12 @@ XS(XS_error) {
         psgi_check_args(0);
 
 	if (uwsgi.threads > 1) {
-        	ST(0) = sv_bless(newRV(sv_newmortal()), ((HV **)wi->error)[wsgi_req->async_id]);
+        	ST(0) = sv_bless(newRV_noinc(newSV(0)), ((HV **)wi->error)[wsgi_req->async_id]);
 	}
 	else {
-        	ST(0) = sv_bless(newRV(sv_newmortal()), ((HV **)wi->error)[0]);
+        	ST(0) = sv_bless(newRV_noinc(newSV(0)), ((HV **)wi->error)[0]);
 	}
+        sv_2mortal(ST(0));
         XSRETURN(1);
 }
 
@@ -41,11 +42,12 @@ XS(XS_input) {
         psgi_check_args(0);
 
 	if (uwsgi.threads > 1) {
-        	ST(0) = sv_bless(newRV(sv_newmortal()), ((HV **)wi->input)[wsgi_req->async_id]);
+        	ST(0) = sv_bless(newRV_noinc(newSV(0)), ((HV **)wi->input)[wsgi_req->async_id]);
 	}
 	else {
-        	ST(0) = sv_bless(newRV(sv_newmortal()), ((HV **)wi->input)[0]);
+        	ST(0) = sv_bless(newRV_noinc(newSV(0)), ((HV **)wi->input)[0]);
 	}
+        sv_2mortal(ST(0));
         XSRETURN(1);
 }
 
@@ -54,7 +56,7 @@ XS(XS_psgix_logger) {
 	psgi_check_args(1);
 	HV *hv_args = (HV *) (SvRV(ST(0)));
 	if (!hv_exists(hv_args, "level", 5) || !hv_exists(hv_args, "message", 7)) {
-		Perl_croak(aTHX_ "psgix.logger requires bot level and message items");
+		Perl_croak(aTHX_ "psgix.logger requires both level and message items");
 	}
 	char *level = SvPV_nolen(*(hv_fetch(hv_args, "level", 5, 0)));
 	char *message = SvPV_nolen(*(hv_fetch(hv_args, "message", 7, 0)));
@@ -80,11 +82,12 @@ XS(XS_stream)
 
 		SvREFCNT_dec(response);
 		if (uwsgi.threads > 1) {
-                	ST(0) = sv_bless(newRV(sv_newmortal()), ((HV **)wi->stream)[wsgi_req->async_id]);
+                	ST(0) = sv_bless(newRV_noinc(newSV(0)), ((HV **)wi->stream)[wsgi_req->async_id]);
 		}
 		else {
-                	ST(0) = sv_bless(newRV(sv_newmortal()), ((HV **)wi->stream)[0]);
+                	ST(0) = sv_bless(newRV_noinc(newSV(0)), ((HV **)wi->stream)[0]);
 		}
+                sv_2mortal(ST(0));
                 XSRETURN(1);
 	}
 	else {
@@ -108,7 +111,7 @@ XS(XS_input_read) {
         unsigned long arg_len = SvIV(ST(2));
 
 	long offset = 0;
-	if (items > 2) {
+	if (items > 3) {
 		offset = (long) SvIV(ST(3));
 	}
 
@@ -271,6 +274,51 @@ nonworker:
 	newCONSTSUB(stash, "SPOOL_RETRY", newSViv(-1));
 	newCONSTSUB(stash, "SPOOL_IGNORE", newSViv(0));
 
+	HV *_opts = newHV();
+
+	int i;
+	for (i = 0; i < uwsgi.exported_opts_cnt; i++) {
+		if (hv_exists(_opts, uwsgi.exported_opts[i]->key, strlen(uwsgi.exported_opts[i]->key))) {
+			SV **value = hv_fetch(_opts, uwsgi.exported_opts[i]->key, strlen(uwsgi.exported_opts[i]->key), 0);
+			// last resort !!!
+			if (!value) {
+				uwsgi_log("[perl] WARNING !!! unable to build uwsgi::opt hash !!!\n");
+				goto end;
+			}
+			if (SvTYPE(SvRV(*value)) == SVt_PVAV) {
+				if (uwsgi.exported_opts[i]->value == NULL) {
+                                        av_push((AV *)SvRV(*value), newSViv(1));
+                                }
+                                else {
+                                        av_push((AV *)SvRV(*value), newSVpv(uwsgi.exported_opts[i]->value, 0));
+                                }
+			}
+			else {
+				AV *_opt_a = newAV();
+				av_push(_opt_a, SvREFCNT_inc(*value));
+				if (uwsgi.exported_opts[i]->value == NULL) {
+					av_push(_opt_a, newSViv(1));
+				}
+				else {
+					av_push(_opt_a, newSVpv(uwsgi.exported_opts[i]->value, 0));
+				}
+				hv_store(_opts, uwsgi.exported_opts[i]->key, strlen(uwsgi.exported_opts[i]->key), newRV_inc((SV *) _opt_a), 0);
+			}
+		}
+		else {
+			if (uwsgi.exported_opts[i]->value == NULL) {
+				hv_store(_opts, uwsgi.exported_opts[i]->key, strlen(uwsgi.exported_opts[i]->key), newSViv(1), 0);
+			}
+			else {
+				hv_store(_opts, uwsgi.exported_opts[i]->key, strlen(uwsgi.exported_opts[i]->key), newSVpv(uwsgi.exported_opts[i]->value, 0), 0);
+			}
+		}
+	}
+
+	newCONSTSUB(stash, "opt", newRV_inc((SV *) _opts));
+
+end:
+
         init_perl_embedded_module();
 
 }
@@ -370,12 +418,16 @@ int init_psgi_app(struct wsgi_request *wsgi_req, char *app, uint16_t app_len, Pe
 		// uperl.embedding as an argument so we won't execute
 		// BEGIN blocks in app_name twice.
 		{
-			char *perl_init_arg[] = { "", "-e", "0" };
+			char *perl_e_arg = uwsgi_concat2("#line 0 ", app_name);
+			char *perl_init_arg[] = { "", "-e", perl_e_arg };
 			if (perl_parse(interpreters[i], xs_init, 3, perl_init_arg, NULL)) {
 				// what to do here ? i hope no-one will use threads with dynamic apps... but clear the whole stuff...
 				free(callables);
+                                free(perl_e_arg);
 				uwsgi_perl_free_stashes();
 				goto clear;
+			} else {
+				free(perl_e_arg);
 			}
 		}
 
