@@ -397,6 +397,7 @@ void uwsgi_python_post_fork() {
 	PyErr_Clear();
 
 	if (uwsgi.mywid > 0) {
+		uwsgi_python_set_thread_name(0);
 		if (up.auto_reload) {
 			// spawn the reloader thread
 			pthread_t par_tid;
@@ -1270,11 +1271,39 @@ void uwsgi_python_enable_threads() {
 		up.reset_ts = threaded_reset_ts;
 	}
 
+	
+
 	uwsgi_log("python threads support enabled\n");
 	
 
 }
 
+void uwsgi_python_set_thread_name(int core_id) {
+	// call threading.currentThread (taken from mod_wsgi, but removes DECREFs as thread in uWSGI are fixed)
+	PyObject *threading_module = PyImport_ImportModule("threading");
+        if (threading_module) {
+                PyObject *threading_module_dict = PyModule_GetDict(threading_module);
+                if (threading_module_dict) {
+#ifdef PYTHREE
+                        PyObject *threading_current = PyDict_GetItemString(threading_module_dict, "current_thread");
+#else
+                        PyObject *threading_current = PyDict_GetItemString(threading_module_dict, "currentThread");
+#endif
+                        if (threading_current) {
+                                PyObject *current_thread = PyEval_CallObject(threading_current, (PyObject *)NULL);
+                                if (!current_thread) {
+                                        // ignore the error
+                                        PyErr_Clear();
+                                }
+                                else {
+                                        PyObject_SetAttrString(current_thread, "name", PyString_FromFormat("uWSGIWorker%dCore%d", uwsgi.mywid, core_id));
+                                        Py_INCREF(current_thread);
+                                }
+                        }
+                }
+        }
+}
+
 void uwsgi_python_init_thread(int core_id) {
 
 	// set a new ThreadState for each thread
@@ -1286,29 +1315,7 @@ void uwsgi_python_init_thread(int core_id) {
 	uwsgi_log("python ThreadState %d = %p\n", core_id, pts);
 #endif
 	UWSGI_GET_GIL;
-	// call threading.currentThread (taken from mod_wsgi, but removes DECREFs as thread in uWSGI are fixed)
-	PyObject *threading_module = PyImport_ImportModule("threading");
-        if (threading_module) {
-        	PyObject *threading_module_dict = PyModule_GetDict(threading_module);
-                if (threading_module_dict) {
-#ifdef PYTHREE
-			PyObject *threading_current = PyDict_GetItemString(threading_module_dict, "current_thread");
-#else
-			PyObject *threading_current = PyDict_GetItemString(threading_module_dict, "currentThread");
-#endif
-                        if (threading_current) {
-                                PyObject *current_thread = PyEval_CallObject(threading_current, (PyObject *)NULL);
-                                if (!current_thread) {
-					// ignore the error
-                                        PyErr_Clear();
-                                }
-				else {
-					PyObject_SetAttrString(current_thread, "name", PyString_FromFormat("uWSGIWorker%dCore%d", uwsgi.mywid, core_id));
-					Py_INCREF(current_thread);
-				}
-                        }
-                }
-        }
+	uwsgi_python_set_thread_name(core_id);
 	UWSGI_RELEASE_GIL;
 	
 
@@ -1418,7 +1425,15 @@ void *uwsgi_python_autoreloader_thread(void *foobar) {
 			int found = 0;
 			struct uwsgi_string_list *usl = up.auto_reload_ignore;
 			while(usl) {
+#ifdef PYTHREE
+				PyObject *zero = PyUnicode_AsUTF8String(mod_name);
+				char *str_mod_name = PyString_AsString(zero);
+				int ret_cmp = strcmp(usl->value, str_mod_name);
+				Py_DECREF(zero);
+				if (!ret_cmp) {
+#else
 				if (!strcmp(usl->value, PyString_AsString(mod_name))) {
+#endif
 					found = 1;
 					break;
 				}
